/**
 * @file LCD_mini_click.c
 * @brief <h2> LCD mini click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * LCD mini click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page Example
 * @date        14 Dec 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    STM32F107VC
 * -<b> Dev. Board    </b> :    EasyMx PRO v7
 * -<b> Ext. Modules  </b> :    LCD mini click
 * -<b> SW            </b> :    mikroC PRO for ARM v.4.9.0
 *
 */

/**
 * @page Dev. board setup
 *
 * 1) Put LCD mini click into mikroBUS 1 slot.
 *
 */


/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>

/******************************************************************************
* Preprocessors
*******************************************************************************/


/******************************************************************************
* Typedefs
*******************************************************************************/


/******************************************************************************
* Variable Definitions
*******************************************************************************/
sbit DigPot_CS      at GPIOA_ODR.B4;

sbit SPExpanderRST  at GPIOC_ODR.B2;
sbit SPExpanderCS   at GPIOD_ODR.B13;

static uint8_t value = 0x00;

static unsigned int pwm_period;

/******************************************************************************
* Function Prototypes
*******************************************************************************/

static void system_init( void );
static void set_contrast( uint8_t value );
static void set_bcklight( uint8_t value );

/******************************************************************************
* Private Functions
*******************************************************************************/

/**************************************************************************//**
 *
 */
static void system_init( void )
{
    GPIO_Digital_Output( &GPIOA_ODR, _GPIO_PINMASK_4 );
    DigPot_CS = 1;
    
    // Initialise SPI.
    SPI3_Init_Advanced(
        _SPI_FPCLK_DIV64,
        _SPI_MASTER |
        _SPI_8_BIT |
        _SPI_CLK_IDLE_LOW |
        _SPI_FIRST_CLK_EDGE_TRANSITION |
        _SPI_MSB_FIRST |
        _SPI_SS_DISABLE |
        _SPI_SSM_ENABLE |
        _SPI_SSI_1, &_GPIO_MODULE_SPI3_PC10_11_12
    );
    
    SPI_Lcd_Config(0);                     // Initialize Lcd over SPI interface
    SPI_Lcd_Cmd(_LCD_CLEAR);               // Clear display
    SPI_Lcd_Cmd(_LCD_CURSOR_OFF);          // Turn cursor off
    
    // Initialise PWM for controlling back light.
    pwm_period = PWM_TIM5_Init(500);
    PWM_TIM5_Set_Duty(0x00, _PWM_NON_INVERTED, _PWM_CHANNEL1);
    PWM_TIM5_Start(_PWM_CHANNEL1, &_GPIO_MODULE_TIM5_CH1_PA0);
}

/**************************************************************************//**
 *
 */
static void set_contrast( uint8_t value )
{
    DigPot_CS = 0;

    SPI3_Write(0x00);
    SPI3_Write(~value);

    DigPot_CS = 1;
}

/**************************************************************************//**
 *
 */
static void set_bcklight( uint8_t value )
{
    PWM_TIM5_Set_Duty(
        (uint16_t) floor(value / 255.0f * pwm_period),
        _PWM_NON_INVERTED, 
        _PWM_CHANNEL1
    );
}

/******************************************************************************
* Public Functions
*******************************************************************************/

void main()
{
    system_init();
    
    SPI_Lcd_Out(1, 6, "mikroE");
    SPI_Lcd_Out(2, 2, "LCD mini click");

    set_bcklight(0xFF);
    set_contrast(0xDF);
    
    Delay_ms(5000);

    while (1)
    {
        set_bcklight(value);
        set_contrast(value);
        
        value++;
        delay_ms(40);
    }
}

/*************** END OF FUNCTIONS *********************************************/